"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const server_1 = require("@simplewebauthn/server");
const webAuthnService = require("../service");
const server_settings_1 = require("../../../server-settings");
class WebAuthnAttestationController extends controller_1.default {
    async get(ctx) {
        const user = ctx.session.registerUser;
        const attestationOptions = server_1.generateAttestationOptions({
            serviceName: server_settings_1.getSetting('webauthn.serviceName'),
            rpID: server_settings_1.getSetting('webauthn.relyingPartyId', new URL(process.env.PUBLIC_URI).host),
            userID: user.id.toString(),
            userName: user.nickname,
            timeout: 60000,
            attestationType: 'direct',
            excludedCredentialIDs: (await webAuthnService.findDevicesByUser(user)).map(device => device.credentialID),
            /**
             * The optional authenticatorSelection property allows for specifying more constraints around
             * the types of authenticators that users to can use for attestation
             */
            authenticatorSelection: {
                authenticatorAttachment: 'cross-platform',
                userVerification: 'preferred',
                requireResidentKey: false,
            },
        });
        ctx.session.webAuthnChallengeRegister = attestationOptions.challenge;
        ctx.response.body = attestationOptions;
    }
    async post(ctx) {
        const user = ctx.session.registerUser;
        const body = ctx.request.body;
        const expectedChallenge = ctx.session.webAuthnChallengeRegister;
        ctx.session.webAuthnChallengeRegister = null;
        let verification;
        try {
            verification = await server_1.verifyAttestationResponse({
                credential: body,
                expectedChallenge,
                expectedOrigin: server_settings_1.getSetting('webauthn.expectedOrigin', new URL(process.env.PUBLIC_URI).origin),
                expectedRPID: server_settings_1.getSetting('webauthn.relyingPartyId', new URL(process.env.PUBLIC_URI).host),
            });
        }
        catch (error) {
            /* eslint-disable-next-line no-console */
            console.error(error);
            ctx.status = 400;
            ctx.response.body = { error: error.message };
            return;
        }
        const { verified, authenticatorInfo } = verification;
        if (verified) {
            const { base64PublicKey, base64CredentialID, counter } = authenticatorInfo;
            const existingDevice = (await webAuthnService.findDevicesByUser(user)).find(device => device.credentialID === base64CredentialID);
            if (!existingDevice) {
                await webAuthnService.save({
                    user,
                    credentialID: base64CredentialID,
                    publicKey: base64PublicKey,
                    counter,
                });
                ctx.session.registerUser = null;
            }
        }
        ctx.response.body = { verified };
    }
}
exports.default = new WebAuthnAttestationController();
//# sourceMappingURL=attestation.js.map