"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const privilegeService = require("../../privilege/service");
const hal = require("../formats/hal");
const http_errors_1 = require("@curveball/http-errors");
const service_1 = require("../service");
const userService = require("../../user/service");
const crypto = require("crypto");
const bcrypt = require("bcrypt");
class ClientCollectionController extends controller_1.default {
    async get(ctx) {
        const user = await userService.findById(+ctx.params.id);
        if (user.id !== ctx.state.user.id) {
            if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
                throw new http_errors_1.Forbidden('Only users with the "admin" privilege can inspect OAuth2 clients that are not your own');
            }
        }
        const clients = await service_1.findByUser(user);
        ctx.response.body = hal.collection(user, clients);
    }
    async post(ctx) {
        const user = await userService.findById(+ctx.params.id);
        if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
            throw new http_errors_1.Forbidden('Only users with the "admin" privilege can inspect OAuth2 clients that are not your own');
        }
        const allowedGrantTypes = [];
        if (ctx.request.body.allowClientCredentials) {
            allowedGrantTypes.push('client_credentials');
        }
        if (ctx.request.body.allowAuthorizationCode) {
            allowedGrantTypes.push('authorization_code');
        }
        if (ctx.request.body.allowImplicit) {
            allowedGrantTypes.push('implicit');
        }
        if (ctx.request.body.allowRefreshToken) {
            allowedGrantTypes.push('refresh_token');
        }
        if (ctx.request.body.allowPassword) {
            allowedGrantTypes.push('password');
        }
        let clientId = ctx.request.body.clientId;
        const redirectUris = ctx.request.body.redirectUris.trim().split(/\r\n|\n/).filter((line) => !!line);
        if (!clientId) {
            clientId = randomId(10);
        }
        else if (clientId.length < 6) {
            throw new http_errors_1.UnprocessableEntity('clientId must be at least 6 characters or left empty');
        }
        if (!allowedGrantTypes) {
            throw new http_errors_1.UnprocessableEntity('You must specify the allowedGrantTypes property');
        }
        const clientSecret = randomId(20);
        const newClient = {
            clientId,
            user,
            allowedGrantTypes: allowedGrantTypes,
            clientSecret: await bcrypt.hash(clientSecret, 12),
        };
        const client = await service_1.create(newClient, redirectUris);
        ctx.response.body = hal.newClientSuccess(client, redirectUris, clientSecret);
    }
}
function randomId(bytes) {
    const buff = crypto.randomBytes(bytes);
    return buff.toString('base64').replace(/\+/g, '-').replace(/\//g, '_').replace(/=/g, '');
}
exports.default = new ClientCollectionController();
//# sourceMappingURL=collection.js.map