"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathName = exports.recordToModel = exports.hasTotp = exports.validateTotp = exports.validatePassword = exports.updatePassword = exports.createPassword = exports.save = exports.findByHref = exports.findByIdentity = exports.hasUsers = exports.findActiveById = exports.findById = exports.findAll = exports.fieldNames = exports.InactiveUser = void 0;
const http_errors_1 = require("@curveball/http-errors");
const bcrypt = require("bcrypt");
const otplib = require("otplib");
const database_1 = require("../database");
class InactiveUser extends Error {
}
exports.InactiveUser = InactiveUser;
exports.fieldNames = [
    'id',
    'identity',
    'nickname',
    'created',
    'type',
    'active'
];
async function findAll() {
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM users`;
    const result = await database_1.default.query(query);
    const users = [];
    for (const user of result[0]) {
        users.push(recordToModel(user));
    }
    return users;
}
exports.findAll = findAll;
async function findById(id) {
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM users WHERE id = ?`;
    const result = await database_1.default.query(query, [id]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound('User with id: ' + id + ' not found');
    }
    return recordToModel(result[0][0]);
}
exports.findById = findById;
async function findActiveById(id) {
    const user = await findById(id);
    if (!user.active) {
        throw new InactiveUser('User with identity ' + user.identity + ' is not active');
    }
    return user;
}
exports.findActiveById = findActiveById;
/**
 * Returns true if more than 1 user exists in the system.
 *
 * If there are 0 users, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
async function hasUsers() {
    const query = 'SELECT 1 FROM users LIMIT 1';
    const result = await database_1.default.query(query);
    return result[0].length > 0;
}
exports.hasUsers = hasUsers;
async function findByIdentity(identity) {
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM users WHERE identity = ?`;
    const result = await database_1.default.query(query, [identity]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound('User with identity: ' + identity + ' not found');
    }
    return recordToModel(result[0][0]);
}
exports.findByIdentity = findByIdentity;
/**
 * Finds a user by its href'.
 *
 * This can be a string like /user/1, or a full url.
 * It can also be the uri listed in the 'identity' field.
 */
async function findByHref(href) {
    const pathName = getPathName(href);
    const matches = pathName.match(/^\/user\/([0-9]+)$/);
    if (!matches) {
        return findByIdentity(href);
    }
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM users WHERE id = ?`;
    const result = await database_1.default.query(query, [matches[1]]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound('User with href: ' + href + ' not found');
    }
    return recordToModel(result[0][0]);
}
exports.findByHref = findByHref;
async function save(user) {
    if (!isExistingUser(user)) {
        // New user
        const query = 'INSERT INTO users SET ?, created = UNIX_TIMESTAMP()';
        const newUserRecord = {
            identity: user.identity,
            nickname: user.nickname,
            type: userTypeToInt(user.type),
            active: user.active ? 1 : 0
        };
        const result = await database_1.default.query(query, [newUserRecord]);
        return {
            id: result[0].insertId,
            ...user
        };
    }
    else {
        // Update user
        const query = 'UPDATE users SET ? WHERE id = ?';
        const updateUserRecord = {
            identity: user.identity,
            nickname: user.nickname,
        };
        await database_1.default.query(query, [updateUserRecord, user.id]);
        return user;
    }
}
exports.save = save;
async function createPassword(user, password) {
    const query = 'INSERT INTO user_passwords SET user_id = ?, password = ?';
    await database_1.default.query(query, [
        user.id,
        await bcrypt.hash(password, 12)
    ]);
}
exports.createPassword = createPassword;
async function updatePassword(user, password) {
    const query = 'UPDATE user_passwords SET password = ? WHERE user_id = ?';
    await database_1.default.query(query, [
        await bcrypt.hash(password, 12),
        user.id
    ]);
}
exports.updatePassword = updatePassword;
/**
 * Returns true or false if the password was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validatePassword(user, password) {
    const query = 'SELECT password FROM user_passwords WHERE user_id = ?';
    const result = await database_1.default.query(query, [user.id]);
    const hashes = result[0].map((row) => row.password);
    for (const hash of hashes) {
        if (await bcrypt.compare(password, hash.toString('utf-8'))) {
            return true;
        }
    }
    return false;
}
exports.validatePassword = validatePassword;
/**
 * Returns true or false if the totp token was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validateTotp(user, token) {
    const query = 'SELECT secret FROM user_totp WHERE user_id = ?';
    const result = await database_1.default.query(query, [user.id]);
    if (!result[0].length) {
        // Not set up
        return false;
    }
    const secret = result[0][0].secret;
    return otplib.authenticator.check(token, secret);
}
exports.validateTotp = validateTotp;
/**
 * Returns true or false if the totp was provided or not.
 *
 *
 */
async function hasTotp(user) {
    const query = 'SELECT secret FROM user_totp WHERE user_id = ?';
    const result = await database_1.default.query(query, [user.id]);
    return result[0].length !== 0;
}
exports.hasTotp = hasTotp;
function userTypeIntToUserType(input) {
    switch (input) {
        case 1: return 'user';
        case 2: return 'app';
        case 3: return 'group';
        default:
            throw new Error('Unknown user type id: ' + input);
    }
}
function userTypeToInt(input) {
    switch (input) {
        case 'user': return 1;
        case 'app': return 2;
        case 'group': return 3;
    }
}
function recordToModel(user) {
    return {
        id: user.id,
        identity: user.identity,
        nickname: user.nickname,
        created: new Date(user.created * 1000),
        type: userTypeIntToUserType(user.type),
        active: !!user.active
    };
}
exports.recordToModel = recordToModel;
function isExistingUser(user) {
    return user.id !== undefined;
}
function getPathName(href) {
    let url;
    try {
        url = new URL(href);
    }
    catch {
        return href;
    }
    return url.pathname;
}
exports.getPathName = getPathName;
//# sourceMappingURL=service.js.map